@@ -150,7 +150,7 @@ def getbusdata(city,keywords,accurate=True):
         url = 'http://map.baidu.com/?qt=bsl&uid='+uid+'&c='+c
         response = urllib.request.urlopen(url,timeout = 60)
         searchinfo=json.loads(response.read().decode('utf8'))
-        linename = searchinfo['content'][0]['name']
+        linename = searchinfo['content'][0]['pair_line']['name']
         stations = searchinfo['content'][0]['stations']
         geo = searchinfo['content'][0]['geo'].split('|')[2][:-1].split(',')
         stationgeo = []
@@ -189,6 +189,7 @@ def getbusdata(city,keywords,accurate=True):
         for uid in getlineuid(keyword,c,accurate):
             if uid not in uids:
                 try:
+                    print(uid,c)
                     linename,coo,stationnames,stationgeo = getlinegeo(uid,c)
                     coo = pd.DataFrame(list(pd.DataFrame(coo)[0].str.split(',')))
                     coo[0],coo[1] = bd09towgs84(coo[0],coo[1])
